<?php
/**
 * Author: topnuomi
 * Date: 2021/1/8/0008 14:32
 */

namespace top\base;

/**
 * Class Application
 * @property Attribute attribute
 * @property Application app
 * @property Config config
 * @property Route route
 * @property Request request
 * @property Response response
 * @property Middleware middleware
 * @package top
 */
class Application extends Container
{

    /**
     * 应用命名空间
     * @var string
     */
    protected string $namespace = '';

    /**
     * 应用名称
     * @var string
     */
    protected string $name = '';

    /**
     * 应用目录
     * @var string
     */
    protected string $path = '';

    /**
     * 绑定标识
     * @var array
     */
    protected array $bind = [
        'attribute'   => Attribute::class,
        'application' => Application::class,
        'app'         => 'application',
        'config'      => Config::class,
        'route'       => Route::class,
        'request'     => Request::class,
        'response'    => Response::class,
        'middleware'  => Middleware::class,
    ];

    /**
     * Application constructor.
     */
    public function __construct()
    {
        // 实例化后保证从容器中取出单例
        static::setInstance($this);

        // 将当前实例放入容器实例池，以实现全局调用同一个实例
        $this->instance('app', $this);

        // 初始化应用
        $this->init();
    }

    /**
     * 初始化应用
     */
    private function init(): void
    {
        // 加载系统函数库
        require __DIR__ . '/functions/common.php';
    }

    /**
     * 设置/获取命名空间
     * @param string $namespace
     * @return string|$this
     */
    public function namespace(string $namespace = ''): string|static
    {
        if (!$this->namespace) {
            $this->namespace = $namespace;
            return $this;
        }
        return $this->namespace;
    }

    /**
     * 设置应用名称
     * @param string $name
     * @return string|Application
     */
    public function name(string $name = ''): string|static
    {
        if (!$this->name) {
            $this->name = $name;
            return $this;
        }

        return $this->name;
    }

    /**
     * 设置应用目录
     * @param string $path
     * @return string|Application
     */
    public function path(string $path = ''): string|static
    {
        if (!$this->path) {
            $this->path = $path;
            return $this;
        }

        return $this->path;
    }

    /**
     * 启动应用
     */
    public function boot(): void
    {
        // $response         = app()->route->dispatch();
        // $responseInstance = app()->response;
        // if ($response instanceof $responseInstance) {
        //     echo $responseInstance->dispatch($response);
        // } else {
        //     echo $response;
        // }
    }

}
