<?php
/**
 * Author: topnuomi
 * Date: 2021/1/8/0008 14:20
 */

namespace top\base;

use ReflectionClass;
use ReflectionException;

class Attribute
{

    /**
     * 反射类池
     * @var array
     */
    private array $refClasses = [];

    /**
     * 获取到的类注解
     * @var array
     */
    private array $classAttributes = [];

    /**
     * 获取到的方法注解
     * @var array
     */
    private array $methodAttributes = [];

    /**
     * 获取类注解
     * @param string $className
     * @param string|null $attributeName
     * @return mixed
     * @throws ReflectionException
     */
    public function classAttributes(string $className, ?string $attributeName = null): mixed
    {
        if (!isset($this->classAttributes[$className])) {
            $refClass = self::getClassRef($className);
            foreach ($refClass->getAttributes() as $classAttribute) {
                $this->classAttributes[$className][$classAttribute->getName()] = $classAttribute->newInstance();
            }
        }

        if (null === $attributeName) {
            return $this->classAttributes[$className] ?? null;
        } elseif (isset($this->classAttributes[$className][$attributeName])) {
            return $this->classAttributes[$className][$attributeName];
        }

        return null;
    }

    /**
     * 获取方法注解
     * @param string $className
     * @param string $methodName
     * @param string|null $attributeName
     * @return mixed
     * @throws ReflectionException
     */
    public function methodAttributes(string $className, string $methodName, ?string $attributeName = null): mixed
    {
        if (!isset($this->methodAttributes[$className][$methodName])) {
            $refClass  = self::getClassRef($className);
            $refMethod = $refClass->getMethod($methodName);
            foreach ($refMethod->getAttributes() as $methodAttribute) {
                $this->methodAttributes[$className][$methodName][$methodAttribute->getName()] = $methodAttribute->newInstance();
            }
        }

        if (null === $attributeName) {
            return $this->methodAttributes[$className][$methodName] ?? null;
        } elseif (isset($this->methodAttributes[$className][$methodName][$attributeName])) {
            return $this->methodAttributes[$className][$methodName][$attributeName];
        }

        return null;
    }

    /**
     * 获取类反射
     * @param $className
     * @return ReflectionClass
     * @throws ReflectionException
     */
    private function getClassRef($className): ReflectionClass
    {
        if (!isset($this->refClasses[$className])) {
            try {
                $this->refClasses[$className] = new ReflectionClass($className);
            } catch (ReflectionException $e) {
                throw $e;
            }
        }

        return $this->refClasses[$className];
    }

}
