<?php
/**
 * Author: topnuomi
 * Date: 2021/3/13 11:34
 */

namespace top\base;

use ReflectionException;
use top\exceptions\app\CallFailed;
use top\exceptions\app\ClassNotFound;
use top\exceptions\app\ClassNotInstantiable;
use top\exceptions\app\DefaultValueMissing;

/**
 * 门面
 * Class Facade
 * @package top\base
 */
class Facade
{
    /**
     * 代表类名
     * @return string
     */
    public static function getRepresentName(): string
    {
        return static::class;
    }

    /**
     * 构造类实例
     * @return mixed
     * @throws CallFailed
     * @throws ClassNotFound
     */
    private static function makeInstance(): mixed
    {
        return Container::getInstance()->make(static::getRepresentName());
    }

    /**
     * 执行方法
     * @param string $name
     * @param array $arguments
     * @return mixed
     * @throws CallFailed
     */
    private static function invoke(string $name, array $arguments = []): mixed
    {
        try {
            $instance = self::makeInstance();
            return Container::getInstance()->invokeMethod($instance, $name, $arguments);
        } catch (CallFailed | ClassNotFound $exception) {
            throw new CallFailed($exception->getMessage());
        }
    }

    /**
     * 方法静态调用
     * @param string $name
     * @param array $arguments
     * @return mixed
     * @throws CallFailed
     */
    public static function __callStatic(string $name, array $arguments): mixed
    {
        return self::invoke($name, $arguments);
    }

    /**
     * 方法动态调用
     * @param $name
     * @param $arguments
     * @return mixed
     * @throws CallFailed
     */
    public function __call($name, $arguments)
    {
        return self::invoke($name, $arguments);
    }
}
