<?php
/**
 * Author: topnuomi
 * Date: 2021/2/24 17:11
 */

namespace top\base;

use Closure;
use top\exceptions\app\CallFailed;
use top\exceptions\BaseException;

/**
 * 中间件
 * Class Middleware
 * @package top\base
 */
class Middleware
{

    /**
     * Middleware constructor.
     * @param Application $application
     */
    public function __construct(
        private Application $application,
    )
    {
    }

    /**
     * 中间件
     * @var array
     */
    private array $middlewares = [];

    /**
     * 添加中间件
     * @param string|array $middlewares
     */
    public function addMiddleware(string|array $middlewares)
    {
        if (is_array($middlewares)) {
            $this->middlewares = array_merge($this->middlewares, $middlewares);
        } else {
            $this->middlewares[] = $middlewares;
        }
    }

    /**
     * 执行中间件
     * @param Closure $application
     * @return Response
     * @throws CallFailed
     */
    public function handler(Closure $application): Response
    {
        // 加载全局配置文件中配置的中间件
        $middlewares = array_reverse($this->middlewares);

        $requestInstance  = $this->application->request;
        $responseInstance = $this->application->response;
        // 打包应用
        $next = $application;
        foreach ($middlewares as $middleware) {
            $next = function () use ($requestInstance, $responseInstance, $middleware, $next) {
                $response = (new $middleware)->handler($requestInstance, $next);
                if ($response instanceof $responseInstance) {
                    return $response;
                } else {
                    throw new BaseException(sprintf("中间件必须返回 '%s' 实例", $responseInstance::class));
                }
            };
        }

        return $this->application->invokeFunction($next);
    }
}
