<?php
/**
 * Author: topnuomi
 * Date: 2021/1/8/0008 14:47
 */

namespace top\base;

/**
 * 请求
 * Class Request
 * @package top
 */
class Request
{
    /**
     * HTTP Header
     * @var array
     */
    private array $header;

    /**
     * SERVER变量
     * @var array
     */
    private array $server;

    /**
     * GET变量
     * @var array
     */
    private array $get;

    /**
     * POST变量
     * @var array
     */
    private array $post;

    /**
     * FILES变量
     * @var array
     */
    private array $files;

    /**
     * Body内容
     * @var string
     */
    private string $raw;

    /**
     * 请求模块
     * @var string
     */
    private string $module = '';

    /**
     * 请求控制器
     * @var string
     */
    private string $controller = '';

    /**
     * 请求方法
     * @var string
     */
    private string $method = '';

    /**
     * 请求参数
     * @var array
     */
    private array $parameters = [];

    /**
     * Request constructor.
     */
    public function __construct()
    {
        $this->header = [];
        $this->server = $_SERVER;
        $this->get    = $_GET;
        $this->post   = $_POST;
        $this->files  = $_FILES;
        $this->raw    = file_get_contents('php://input');

        $this->setMethod($_SERVER['REQUEST_METHOD'] ?? '');
    }

    /**
     * 获取访问地址
     * @return string
     */
    public function url(): string
    {
        return $_SERVER['REQUEST_URI'];
    }

    /**
     * 获取模块名称
     * @return string
     */
    public function getModule(): string
    {
        return $this->module;
    }

    /**
     * 设置模块
     * @param string $module
     * @return $this
     */
    public function setModule(string $module): static
    {
        $this->module = $module;
        return $this;
    }

    /**
     * 获取控制器名称
     * @return string
     */
    public function getController(): string
    {
        return $this->controller;
    }

    /**
     * 设置控制器
     * @param string $controller
     * @return $this
     */
    public function setController(string $controller): static
    {
        $this->controller = $controller;
        return $this;
    }

    /**
     * 获取方法名称
     * @return string
     */
    public function getMethod(): string
    {
        return $this->method;
    }

    /**
     * 设置方法
     * @param string $method
     * @return $this
     */
    public function setMethod(string $method): static
    {
        $this->method = strtolower($method);
        return $this;
    }

    /**
     * 获取请求参数
     * @return array
     */
    public function getParameters(): array
    {
        return $this->parameters;
    }

    /**
     * 设置请求参数
     * @param array $parameters
     */
    public function setParameters(array $parameters): void
    {
        $this->parameters = array_merge($this->parameters, $parameters);
    }

}
