<?php
/**
 * Author: topnuomi
 * Date: 2021/1/8/0008 15:09
 */

namespace top\traits;

use top\base\Container;
use top\exceptions\app\CallFailed;
use top\exceptions\app\ClassNotFound;
use top\exceptions\app\ClassNotInstantiable;
use top\exceptions\app\DefaultValueMissing;

/**
 * 单例
 * Trait Instance
 * @package top\traits
 */
trait Instance
{

    /**
     * 当前实例
     * @var object|null
     */
    private static ?object $instance = null;

    /**
     * @param mixed ...$args
     * @return static
     */
    public static function instance(...$args): static
    {
        if (null === self::$instance) {
            try {
                self::$instance = Container::getInstance()->make(static::class, $args);
            } catch (CallFailed | ClassNotFound | ClassNotInstantiable | DefaultValueMissing) {
                self::$instance = null;
            }
        }

        return self::$instance;
    }

}
